/*
    Rekollect: A note taking application
    Copyright (C) 2009, 2010  Jason Jackson <jaxhoo@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#include "notecollection.h"
#include "note.h"
#include "io/abstractreader.h"
#include "io/native/notereader.h"
#include "io/native/notewriter.h"
#include "datetimeitem.h"

#include <QtCore/QFile>
#include <QtCore/QUuid>
#include <QtGui/QTextCursor>
#include <QtCore/QDateTime>

#include <KStandardDirs>
#include <KSharedConfig>
#include <KConfigGroup>
#include <KLocalizedString>
#include <KSaveFile>
#include <KMessageBox>



NoteCollection::NoteCollection(QObject *parent) :
    QStandardItemModel(parent), m_numberOfDisplayableColumns(2)
{
    setColumnCount(3);
    loadNoteDetails();
    initializeHeader();
    m_nextNewNoteNumber = loadNextNewNoteNumber();
}

void NoteCollection::setWindowCollection(WindowCollection* windowCollection)
{
    m_windowCollection = windowCollection;
}

QString NoteCollection::fileNameAt(const QModelIndex &index) const
{
    QStandardItem *noteData = item(index.row(), FileNameColumn);
    return noteData->text();
}

int NoteCollection::nextNewNoteNumber()
{
    int nextNumber = m_nextNewNoteNumber;
    ++m_nextNewNoteNumber;
    storeNextNewNoteNumber(m_nextNewNoteNumber);
    return nextNumber;
}

Note *NoteCollection::createNewNote(const QString &newNoteName)
{
    QString templateFilePath = KStandardDirs::locate("data", "rekollect/template.xml");
    Note *note = loadNote(templateFilePath);

    if (note) {
        if (newNoteName == QString()) {
            QTextCursor cursor = note->find("%number");
            cursor.insertText(QString("%1").arg(nextNewNoteNumber()));
        }  else {
            QTextCursor cursor(note);
            cursor.select(QTextCursor::BlockUnderCursor);
            cursor.insertText(newNoteName);
        }

        note->updateDocumentName();
        connectNoteSignals(note);
        note->setFileName(createFileName());
        if (saveNote(note)) {
            addNote(note);
            emit noteCreated(note->fileName(), note->documentName());
        }
    }
    return note;
}

Note *NoteCollection::openNote(const QString &fileName)
{
    Note *note = loadNote(fileName);
    if (note) {
        note->updateDocumentName();
        connectNoteSignals(note);
    }
    return note;
}

bool NoteCollection::noteExists(const QString& fileName)
{
    return itemFromFileName(fileName) != 0;
}

int NoteCollection::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return m_numberOfDisplayableColumns;
}

QVariant NoteCollection::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (orientation == Qt::Vertical) {
        return QString();
    } else {
        return QStandardItemModel::headerData(section, orientation, role);
    }
}

void NoteCollection::removeNote(const QString &fileName)
{
    QStandardItem *item = itemFromFileName(fileName);
    if (item != 0) {
        removeRow(item->row());
    }
}

bool NoteCollection::saveNote(Note *note)
{
    return writeNote(note);
}

void NoteCollection::initializeHeader()
{
    setHeaderData(DocumentNameColumn, Qt::Horizontal, i18nc("Note window column header", "Document Name"));
    setHeaderData(ModificationTimeColumn, Qt::Horizontal, i18nc("Note window column header", "Modification Time"));
    setHeaderData(FileNameColumn, Qt::Horizontal, i18nc("Note window column header", "File Name"));
}

void NoteCollection::loadNoteDetails()
{
    QStringList noteList = KGlobal::dirs()->findAllResources("app_notes", QString());
    foreach (const QString &notePath, noteList) {
        Note *note = openNote(notePath);
        if (note) {
            addNote(note);
        }
    }
}

void NoteCollection::addNote(Note *note)
{
    QStandardItem *documentNameItem = new QStandardItem(note->documentName());
    QStandardItem *fileNameItem = new QStandardItem(note->fileName());
    DateTimeItem *modificationTimeItem = new DateTimeItem(note->modificationTime().dateTime());
    QList<QStandardItem *> newRow;
    newRow << documentNameItem << modificationTimeItem << fileNameItem;
    appendRow(newRow);
}

Note *NoteCollection::loadNote(const QString &fileName)
{
    Note *note = new Note(fileName, this);
    QFile file(fileName);
    file.open(QIODevice::ReadOnly);

    QTextCursor noteTextCursor(note);
    NoteReader reader(&noteTextCursor);
    if (!reader.read(&file)) {
        return 0;
    }
    foreach (QString tag, reader.tags()) {
        note->addTag(tag);
    }
    return note;
}

bool NoteCollection::writeNote(Note *note)
{
    KSaveFile saveFile(note->fileName());
    saveFile.open();

    NoteWriter writer(note->rootFrame(), note->tags());
    if (!writer.writeFile(&saveFile)) {
        KMessageBox::error(0,
                           i18n("An error occurred saving the file."),
                           i18n("Save Error"));
                           return false;
    }

    bool isSuccessful = saveFile.finalize();
    saveFile.close();
    emit documentModified(note->fileName(), note->modificationTime().dateTime());
    return isSuccessful;
}

int NoteCollection::loadNextNewNoteNumber() const
{
    KSharedConfigPtr config = KGlobal::config();
    KConfigGroup internalDataGroup = config->group("Internal");
    return internalDataGroup.readEntry("NextUnamedWindow", 1);
}

void NoteCollection::storeNextNewNoteNumber(int nextNote)
{
    KSharedConfigPtr config = KGlobal::config();
    KConfigGroup internalDataGroup = config->group("Internal");
    internalDataGroup.writeEntry("NextUnamedWindow", nextNote);
    config->sync();
}

QStandardItem* NoteCollection::itemFromFileName(const QString &fileName)
{
    QList<QStandardItem *> items = findItems(fileName, Qt::MatchExactly, FileNameColumn);
    if (items.count()) {
        return items.takeFirst();
    } else {
        return 0;
    }

}

void NoteCollection::connectNoteSignals(Note *note)
{
    connect(note, SIGNAL(deleteRequested(QString)), SLOT(removeNote(QString)));
    connect(note, SIGNAL(deleteRequested(QString)), SIGNAL(noteRemoved(QString)));
    connect(note, SIGNAL(documentNameChanged(QString,QString)), SLOT(documentNameChanged(QString,QString)));
    connect(note, SIGNAL(documentNameChanged(QString,QString)), SIGNAL(noteNameChanged(QString,QString)));
    connect(note, SIGNAL(documentModified(QString,QDateTime)), SLOT(documentModified(QString,QDateTime)));
}

QString NoteCollection::createFileName() const
{
    QString newFileName = QUuid::createUuid().toString();
    QString fullFilePath = KStandardDirs::locateLocal("app_notes", newFileName, true);
    if (KStandardDirs::exists(fullFilePath)) {
        return createFileName();
    } else {
        return fullFilePath;
    }
}

void NoteCollection::documentNameChanged(const QString &fileName, const QString &documentName)
{
    QStandardItem *item = itemFromFileName(fileName);
    if (item != 0) {
        QModelIndex indexToChange = index(item->row(), DocumentNameColumn);
        setData(indexToChange, documentName);
    }
}

void NoteCollection::documentModified(const QString &fileName, const QDateTime &modificationTime)
{
    QStandardItem *item = itemFromFileName(fileName);
    if (item != 0) {
        QModelIndex indexToChange = index(item->row(), ModificationTimeColumn);
        setData(indexToChange, modificationTime);
    }
}
