/*
    Rekollect: A note taking application
    Copyright (C) 2009, 2010  Jason Jackson <jaxhoo@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#include "notewindow.h"
#include "noteeditor.h"
#include "note.h"
#include "linkeditdialog.h"
#include "windowcollection.h"

#include <QtGui/QActionGroup>
#include <QtCore/QSignalMapper>
#include <QtCore/QFileInfo>

#include <KActionMenu>
#include <KToolBar>
#include <KIcon>
#include <KAction>
#include <KStandardAction>
#include <KLocale>
#include <KMessageBox>
#include <KInputDialog>

NoteWindow::NoteWindow(Note *note, WindowCollection *parentCollection, QWidget *parent)
    : KMainWindow(parent), m_parentCollection(parentCollection)
{
    resize(640, 480);
    setCaption(note->documentName());
    m_noteEditor = new NoteEditor(note, this);
    note->setIndentWidth(24);
    connect(m_noteEditor, SIGNAL(linkTriggered(QString,QString)), SIGNAL(linkTriggered(QString,QString)));
    connect(note, SIGNAL(documentNameChanged(QString,QString)), SLOT(documentNameChanged(QString,QString)));
    setCentralWidget(m_noteEditor);
    setWindowIcon(KIcon("rekollect"));
    createActions();
    createToolbar();
}

NoteEditor* NoteWindow::editor()
{
    return m_noteEditor;
}

void NoteWindow::removeCurrentLink()
{
    m_noteEditor->removeCurrentLink();
}

void NoteWindow::deleteNote()
{
    if (KMessageBox::questionYesNo(this, i18n("Are you sure you want to delete this note?"),
                                   i18n("Delete Note")) == KMessageBox::Yes) {
        if (m_noteEditor->document()->deleteNote()) {
            close();
        } else {
            KMessageBox::error(this,
                               i18n("An error occurred deleting this note. The note is not deleted."),
                               i18n("Error"));
        }
    }
}

void NoteWindow::editLink()
{
    LinkEditDialog dialog(m_noteEditor->document()->parentNoteCollection(),
                          m_noteEditor->currentLinkText().trimmed(),
                          m_noteEditor->document()->fileName(), this);
                          connect(&dialog, SIGNAL(newNoteRequested(QString)), SLOT(linkToNewNote(QString)));
                          connect(&dialog, SIGNAL(linkToNoteRequested(QString)), SLOT(linkToExistingNote(QString)));
                          dialog.exec();
}

void NoteWindow::closeEvent(QCloseEvent *event)
{
    emit closeRequested(m_noteEditor->document()->fileName());
    KMainWindow::closeEvent(event);
}

void NoteWindow::createActions()
{
    // Initiate automatic document saving
    connect(m_noteEditor, SIGNAL(textChanged()), m_noteEditor->document(), SLOT(updateDocumentName()));

    m_undoAction = KStandardAction::undo(m_noteEditor, SLOT(undo()), this);
    m_undoAction->setEnabled(m_noteEditor->document()->isUndoAvailable());
    connect(m_noteEditor, SIGNAL(undoAvailable(bool)), m_undoAction, SLOT(setEnabled(bool)));

    m_redoAction = KStandardAction::redo(m_noteEditor, SLOT(redo()), this);
    m_redoAction->setEnabled(m_noteEditor->document()->isRedoAvailable());
    connect(m_noteEditor, SIGNAL(redoAvailable(bool)), m_redoAction, SLOT(setEnabled(bool)));

    m_linkUrlAction = new KAction(KIcon("insert-link"), i18n("Link to URL"), this);
    connect(m_linkUrlAction, SIGNAL(triggered()), SLOT(linkToUrl()));

    m_removeLinkAction = new KAction(i18n("Remove Link"), this);
    connect(m_removeLinkAction, SIGNAL(triggered()), m_noteEditor, SLOT(removeCurrentLink()));

    m_boldAction = new KAction(KIcon("format-text-bold"), i18n("Bold"), this);
    m_boldAction->setCheckable(true);
    m_boldAction->setShortcut(Qt::CTRL + Qt::Key_B);
    connect(m_boldAction, SIGNAL(triggered(bool)), m_noteEditor, SLOT(setTextBold(bool)));

    m_italicAction = new KAction(KIcon("format-text-italic"), i18n("Italic"), this);
    m_italicAction->setCheckable(true);
    m_italicAction->setShortcut(Qt::CTRL + Qt::Key_I);
    connect(m_italicAction, SIGNAL(triggered(bool)), m_noteEditor, SLOT(setTextItalic(bool)));

    m_strikeOutAction = new KAction(KIcon("format-text-strikethrough"), i18n("Strike Out"), this);
    m_strikeOutAction->setCheckable(true);
    m_strikeOutAction->setShortcut(Qt::CTRL + Qt::Key_L);
    QFont strikeOutFont;
    strikeOutFont.setStrikeOut(true);
    m_strikeOutAction->setFont(strikeOutFont);
    connect(m_strikeOutAction, SIGNAL(triggered(bool)), m_noteEditor, SLOT(setTextStrikeOut(bool)));

    m_highlightAction = new KAction(i18n("Highlight"), this);
    m_highlightAction->setCheckable(true);
    m_highlightAction->setShortcut(Qt::CTRL + Qt::Key_H);
    connect(m_highlightAction , SIGNAL(triggered(bool)), m_noteEditor, SLOT(setTextHighlight(bool)));

    m_resetFormatAction = new KAction(i18n("Reset Format"), this);
    connect(m_resetFormatAction, SIGNAL(triggered()), m_noteEditor, SLOT(resetTextFormat()));

    m_smallTextAction = new KAction(i18n("S&mall"), this);
    m_smallTextAction->setCheckable(true);
    QFont smallTextFont;
    smallTextFont.setPointSize(Note::SmallFont);
    m_smallTextAction->setFont(smallTextFont);

    m_normalTextAction = new KAction(i18n("&Normal"), this);
    m_normalTextAction->setCheckable(true);
    QFont normalTextFont;
    normalTextFont.setPointSize(Note::NormalFont);
    m_normalTextAction->setFont(normalTextFont);

    m_largeTextAction = new KAction(i18n("&Large"), this);
    m_largeTextAction->setCheckable(true);
    QFont largeTextFont;
    largeTextFont.setPointSize(Note::LargeFont);
    m_largeTextAction->setFont(largeTextFont);

    m_hugeTextAction = new KAction(i18n("&Huge"), this);
    m_hugeTextAction->setCheckable(true);
    QFont hugeTextFont;
    hugeTextFont.setPointSize(Note::HugeFont);
    m_hugeTextAction->setFont(hugeTextFont);

    m_fontSizeSignalMapper = new QSignalMapper(this);
    connect(m_smallTextAction, SIGNAL(triggered()), m_fontSizeSignalMapper, SLOT(map()));
    m_fontSizeSignalMapper->setMapping(m_smallTextAction, Note::SmallFont);
    connect(m_normalTextAction, SIGNAL(triggered()), m_fontSizeSignalMapper, SLOT(map()));
    m_fontSizeSignalMapper->setMapping(m_normalTextAction, Note::NormalFont);
    connect(m_largeTextAction, SIGNAL(triggered()), m_fontSizeSignalMapper, SLOT(map()));
    m_fontSizeSignalMapper->setMapping(m_largeTextAction, Note::LargeFont);
    connect(m_hugeTextAction, SIGNAL(triggered()), m_fontSizeSignalMapper, SLOT(map()));
    m_fontSizeSignalMapper->setMapping(m_hugeTextAction, Note::HugeFont);
    connect(m_fontSizeSignalMapper, SIGNAL(mapped(int)), m_noteEditor, SLOT(setFontSize(int)));

    m_fontSizeGroup = new QActionGroup(this);
    m_fontSizeGroup->addAction(m_smallTextAction);
    m_fontSizeGroup->addAction(m_normalTextAction);
    m_fontSizeGroup->addAction(m_largeTextAction);
    m_fontSizeGroup->addAction(m_hugeTextAction);
    m_normalTextAction->setChecked(true);

    m_increaseFontSizeAction = new KAction(KIcon("format-font-size-more"), i18n("Increase Font Size"), this);
    m_increaseFontSizeAction->setShortcut(Qt::CTRL + Qt::Key_Plus);
    connect(m_increaseFontSizeAction , SIGNAL(triggered()), m_noteEditor, SLOT(increaseFontSize()));

    m_decreaseFontSizeAction = new KAction(KIcon("format-font-size-less"), i18n("Decrease Font Size"), this);
    m_decreaseFontSizeAction->setShortcut(Qt::CTRL + Qt::Key_Minus);
    connect(m_decreaseFontSizeAction , SIGNAL(triggered()), m_noteEditor, SLOT(decreaseFontSize()));

    m_increaseIndentAction = new KAction(KIcon("format-indent-more"), i18n("Increase Indent"), this);
    m_increaseIndentAction->setShortcut(Qt::ALT + Qt::Key_Right);
    connect(m_increaseIndentAction, SIGNAL(triggered()), m_noteEditor, SLOT(indentListMore()));

    m_decreaseIndentAction = new KAction(KIcon("format-indent-less"), i18n("Decrease Indent"), this);
    m_decreaseIndentAction->setShortcut(Qt::ALT + Qt::Key_Left);
    connect(m_decreaseIndentAction, SIGNAL(triggered()), m_noteEditor, SLOT(indentListLess()));

    m_deleteNoteAction = new KAction(KIcon("edit-delete"), i18n("&Delete"), this);
    connect(m_deleteNoteAction, SIGNAL(triggered()), SLOT(deleteNote()));
}

void NoteWindow::createToolbar()
{
    m_searchActionMenu = new KActionMenu(KIcon("edit-find"), i18n("&Search"), this);
    m_searchActionMenu->setDelayed(false);
    m_searchActionMenu->addAction(KStandardAction::find(m_noteEditor, SLOT(slotFind()), this));
    m_searchActionMenu->addAction(KStandardAction::findNext(m_noteEditor, SLOT(slotFindNext()), this));
    toolBar()->addAction(m_searchActionMenu);

    m_linkActionMenu = new KActionMenu(KIcon("insert-link"), i18n("&Link to Note"), this);
    connect(m_linkActionMenu, SIGNAL(triggered()), SLOT(editLink()));
    m_linkActionMenu->addAction(m_linkUrlAction);
    m_linkActionMenu->addAction(m_removeLinkAction);
    toolBar()->addAction(m_linkActionMenu);

    m_formatActionMenu = new KActionMenu(KIcon("preferences-desktop-font"), i18n("&Format"), this);
    m_formatActionMenu->setDelayed(false);
    toolBar()->addAction(m_formatActionMenu);

    m_formatActionMenu->addAction(m_undoAction);
    m_formatActionMenu->addAction(m_redoAction);
    m_formatActionMenu->addSeparator();

    m_formatActionMenu->addAction(m_boldAction);
    m_formatActionMenu->addAction(m_italicAction);
    m_formatActionMenu->addAction(m_strikeOutAction);
    m_formatActionMenu->addAction(m_highlightAction);

    m_formatActionMenu->addSeparator();
    m_formatActionMenu->addAction(m_resetFormatAction);
    m_formatActionMenu->addSeparator();

    m_formatActionMenu->addSeparator()->setText(i18n("Font Size"));
    m_formatActionMenu->addAction(m_smallTextAction);
    m_formatActionMenu->addAction(m_normalTextAction);
    m_formatActionMenu->addAction(m_largeTextAction);
    m_formatActionMenu->addAction(m_hugeTextAction);

    m_formatActionMenu->addAction(m_increaseFontSizeAction);
    m_formatActionMenu->addAction(m_decreaseFontSizeAction);

    m_formatActionMenu->addSeparator();
    m_formatActionMenu->addAction(m_increaseIndentAction);
    m_formatActionMenu->addAction(m_decreaseIndentAction);

    toolBar()->addSeparator();
    toolBar()->addAction(KStandardAction::openNew(this, SIGNAL(newNoteRequested()), this));
    toolBar()->addAction(m_deleteNoteAction);

    connect(m_noteEditor, SIGNAL(currentCharFormatChanged(QTextCharFormat)),
            SLOT(updateFormatMenu(QTextCharFormat)));
}

void NoteWindow::documentNameChanged(const QString &fileName, const QString &documentName)
{
    Q_UNUSED(fileName)
    setWindowTitle(i18n("%0 - Rekollect").arg(documentName));
}

void NoteWindow::linkToUrl()
{
    bool ok;
    QString url = KInputDialog::getText(i18n("Link to URL"),
                                        i18n("Please enter the URL to Link to:"),
                                        QString(), &ok, this);
    if (ok) {
        m_noteEditor->updateLink(url);
    }
}

void NoteWindow::linkToNewNote(const QString& documentName)
{
    QString fileName = m_parentCollection->createNewNoteWindow(documentName);
    linkToExistingNote(fileName);
}

void NoteWindow::linkToExistingNote(const QString& fileName)
{
    QFileInfo fileInfo(fileName);
    QString linkUrl = QString("rekollect:%1").arg(fileInfo.fileName());
    m_noteEditor->updateLink(linkUrl);
}

void NoteWindow::updateFormatMenu(const QTextCharFormat &charFormat)
{
    QTextCharFormat charFormat_ = charFormat;
    QTextCursor cursor = m_noteEditor->textCursor();
    if (cursor.hasSelection() && cursor.position() == cursor.selectionStart()) {
        QTextCursor endCursor(cursor);
        endCursor.setPosition(cursor.selectionEnd());
        charFormat_ = endCursor.charFormat();
    }

    m_boldAction->setChecked(charFormat_.fontWeight() == QFont::Bold);
    m_italicAction->setChecked(charFormat_.fontItalic());
    m_strikeOutAction->setChecked(charFormat_.fontStrikeOut());
    m_highlightAction->setChecked(charFormat_.background() == Qt::yellow);

    switch (static_cast<int>(charFormat.fontPointSize())) {
        case Note::SmallFont:
            m_smallTextAction->setChecked(true);
            break;
        case Note::NormalFont:
            m_normalTextAction->setChecked(true);
            break;
        case Note::LargeFont:
            m_largeTextAction->setChecked(true);
            break;
        case Note::HugeFont:
            m_hugeTextAction->setChecked(true);
            break;
    }
}
